@@ -18,14 +18,14 @@
 //! With the `serde-compat` feature enabled, ts-rs tries parsing serde attributes.  
 //! Please note that not all serde attributes are supported yet.
 
-use std::path::Path;
-use std::{collections::HashMap, fs::OpenOptions};
 use std::{
-    collections::{BTreeMap, BTreeSet, HashSet},
+    any::TypeId,
+    collections::{BTreeMap, BTreeSet, HashMap, HashSet},
+    fs::OpenOptions,
     io::{BufWriter, Write},
+    path::Path,
 };
 
-use std::any::TypeId;
 pub use ts_rs_macros::TS;
 
 #[doc(hidden)]
@@ -36,7 +36,7 @@ pub mod export;
 /// ts-rs comes with implementations for all numeric types, `String`, `Vec`, `Option` and tuples.
 ///
 /// ## get started
-/// [TS](ts_rs::TS) can easily be derived for structs and enums:
+/// [TS](TS) can easily be derived for structs and enums:
 /// ```rust
 /// use ts_rs::TS;
 ///
@@ -46,10 +46,9 @@ pub mod export;
 ///     last_name: String,
 /// }
 /// ```
-/// To actually obtain the bindings, you can call `User::dump` to write the bindings to a file:
+/// To actually obtain the bindings, you can call `User::dump` to write the bindings to a file.
 /// ```rust
 /// # use ts_rs::TS;
-///
 /// # #[derive(TS)]
 /// # struct User {
 /// #     first_name: String,
@@ -59,6 +58,9 @@ pub mod export;
 /// User::dump("bindings.ts").unwrap();
 /// ```
 ///
+/// Preferrably, you should use the [export!](export!) macro, which takes care of dependencies
+/// between types and allows you to decide between `export` and `declare`.
+///
 /// ### struct attributes
 ///
 /// - `#[ts(rename = "..")]`:  
@@ -82,6 +84,9 @@ pub mod export;
 /// - `#[ts(skip)]`:  
 ///   Skip this field  
 ///
+/// - `#[ts(optional)]
+///   Indicates the field may be omitted from the serialized struct
+///
 /// - `#[ts(flatten)]`:  
 ///   Flatten this field (only works if the field is a struct)  
 ///   
@@ -112,6 +117,11 @@ pub trait TS: 'static {
     /// Name of this type in TypeScript.
     fn name() -> String;
 
+    /// Name of this type in TypeScript, with type arguments.
+    fn name_with_type_args(args: Vec<String>) -> String {
+        format!("{}<{}>", Self::name(), args.join(", "))
+    }
+
     /// Formats this types definition in TypeScript, e.g `{ user_id: number }`.
     /// This function will panic if the type cannot be inlined.
     fn inline(#[allow(unused_variables)] indent: usize) -> String {
@@ -209,6 +219,13 @@ macro_rules! impl_proxy {
             fn name() -> String {
                 T::name()
             }
+            fn name_with_type_args(args: Vec<String>) -> String {
+                if args.len() == 1 {
+                    args[0].clone()
+                } else {
+                    format!("[{}]", args.join(", "))
+                }
+            }
             fn inline(indent: usize) -> String {
                 T::inline(indent)
             }
@@ -219,19 +236,122 @@ macro_rules! impl_proxy {
                 T::dependencies()
             }
             fn transparent() -> bool {
-                true
+                T::transparent()
             }
         }
     };
 }
 
 impl_primitives! {
-    u8, i8, u16, i16, u32, i32, u64, i64, f32, f64, usize, isize => "number",
-    u128, i128 => "bigint",
+    u8, i8, u16, i16, u32, i32, f32, f64, usize, isize => "number",
+    u64, i64, u128, i128 => "bigint",
     bool => "boolean",
     String, &'static str => "string",
     () => "null"
 }
+
+#[cfg(feature = "bytes-impl")]
+mod bytes {
+    use std::any::TypeId;
+
+    use super::TS;
+
+    impl TS for bytes::Bytes {
+        fn name() -> String {
+            "Array<number>".to_owned()
+        }
+
+        fn inline(indent: usize) -> String {
+            format!("Array<{}>", u8::inline(indent))
+        }
+
+        fn dependencies() -> Vec<(TypeId, String)> {
+            vec![(TypeId::of::<u8>(), u8::name())]
+        }
+
+        fn transparent() -> bool {
+            true
+        }
+    }
+
+    impl TS for bytes::BytesMut {
+        fn name() -> String {
+            "Array<number>".to_owned()
+        }
+
+        fn inline(indent: usize) -> String {
+            format!("Array<{}>", u8::inline(indent))
+        }
+
+        fn dependencies() -> Vec<(TypeId, String)> {
+            vec![(TypeId::of::<u8>(), u8::name())]
+        }
+
+        fn transparent() -> bool {
+            true
+        }
+    }
+}
+
+#[cfg(feature = "chrono-impl")]
+mod chrono_impls {
+    use std::any::TypeId;
+
+    use chrono::{Date, DateTime, NaiveDate, NaiveDateTime, NaiveTime, TimeZone};
+
+    use super::TS;
+
+    impl_primitives! {
+        NaiveDateTime, NaiveDate, NaiveTime => "string"
+    }
+
+    impl<T: TimeZone + 'static> TS for DateTime<T> {
+        fn name() -> String {
+            "string".to_owned()
+        }
+
+        fn inline(_indent: usize) -> String {
+            "string".to_owned()
+        }
+
+        fn dependencies() -> Vec<(TypeId, String)> {
+            vec![]
+        }
+
+        fn transparent() -> bool {
+            false
+        }
+    }
+
+    impl<T: TimeZone + 'static> TS for Date<T> {
+        fn name() -> String {
+            "string".to_owned()
+        }
+
+        fn inline(_indent: usize) -> String {
+            "string".to_owned()
+        }
+
+        fn dependencies() -> Vec<(TypeId, String)> {
+            vec![]
+        }
+
+        fn transparent() -> bool {
+            false
+        }
+    }
+}
+
+#[cfg(feature = "bigdecimal-impl")]
+impl_primitives! {
+    bigdecimal::BigDecimal => "string"
+}
+
+#[cfg(feature = "uuid-impl")]
+impl_primitives! {
+    uuid::Uuid => "string"
+}
+
 impl_tuples!(T1, T2, T3, T4, T5, T6, T7, T8, T9, T10);
 impl_proxy!(impl<T: TS> TS for Box<T>);
 impl_proxy!(impl<T: TS> TS for std::sync::Arc<T>);
@@ -242,7 +362,12 @@ impl_proxy!(impl<T: TS> TS for std::cell::RefCell<T>);
 
 impl<T: TS> TS for Option<T> {
     fn name() -> String {
-        format!("{} | null", T::name())
+        unreachable!();
+    }
+
+    fn name_with_type_args(args: Vec<String>) -> String {
+        assert_eq!(args.len(), 1);
+        format!("{} | null", args[0])
     }
 
     fn inline(indent: usize) -> String {
@@ -260,11 +385,11 @@ impl<T: TS> TS for Option<T> {
 
 impl<T: TS> TS for Vec<T> {
     fn name() -> String {
-        format!("{}[]", T::name())
+        "Array".to_owned()
     }
 
     fn inline(indent: usize) -> String {
-        format!("{}[]", T::inline(indent))
+        format!("Array<{}>", T::inline(indent))
     }
 
     fn dependencies() -> Vec<(TypeId, String)> {
@@ -278,11 +403,11 @@ impl<T: TS> TS for Vec<T> {
 
 impl<T: TS> TS for HashSet<T> {
     fn name() -> String {
-        format!("{}[]", T::name())
+        "Array".to_owned()
     }
 
     fn inline(indent: usize) -> String {
-        format!("{}[]", T::inline(indent))
+        format!("Array<{}>", T::inline(indent))
     }
 
     fn dependencies() -> Vec<(TypeId, String)> {
@@ -296,11 +421,11 @@ impl<T: TS> TS for HashSet<T> {
 
 impl<T: TS> TS for BTreeSet<T> {
     fn name() -> String {
-        format!("{}[]", T::name())
+        "Array".to_owned()
     }
 
     fn inline(indent: usize) -> String {
-        format!("{}[]", T::inline(indent))
+        format!("Array<{}>", T::inline(indent))
     }
 
     fn dependencies() -> Vec<(TypeId, String)> {
@@ -314,11 +439,11 @@ impl<T: TS> TS for BTreeSet<T> {
 
 impl<K: TS, V: TS> TS for HashMap<K, V> {
     fn name() -> String {
-        format!("{{ [key: {}]: {} }}", K::name(), V::name())
+        "Record".to_owned()
     }
 
     fn inline(indent: usize) -> String {
-        format!("{{ [key: {}]: {} }}", K::inline(indent), V::inline(indent))
+        format!("Record<{}, {}>", K::inline(indent), V::inline(indent))
     }
 
     fn dependencies() -> Vec<(TypeId, String)> {
@@ -335,11 +460,11 @@ impl<K: TS, V: TS> TS for HashMap<K, V> {
 
 impl<K: TS, V: TS> TS for BTreeMap<K, V> {
     fn name() -> String {
-        format!("{{ [key: {}]: {} }}", K::name(), V::name())
+        "Record".to_owned()
     }
 
     fn inline(indent: usize) -> String {
-        format!("{{ [key: {}]: {} }}", K::inline(indent), V::inline(indent))
+        format!("Record<{}, {}>", K::inline(indent), V::inline(indent))
     }
 
     fn dependencies() -> Vec<(TypeId, String)> {
@@ -353,3 +478,21 @@ impl<K: TS, V: TS> TS for BTreeMap<K, V> {
         true
     }
 }
+
+impl<T: TS, const N: usize> TS for [T; N] {
+    fn name() -> String {
+        format!("Array<{}>", T::name())
+    }
+
+    fn inline(indent: usize) -> String {
+        format!("Array<{}>", T::inline(indent))
+    }
+
+    fn dependencies() -> Vec<(TypeId, String)> {
+        vec![(TypeId::of::<T>(), T::name())]
+    }
+
+    fn transparent() -> bool {
+        true
+    }
+}
