@@ -19,6 +19,7 @@ enum ComplexEnum {
     F { nested: SimpleEnum },
     T(i32, SimpleEnum),
 }
+
 #[derive(TS, Deserialize)]
 #[serde(untagged)]
 enum Untagged {
@@ -27,24 +28,25 @@ enum Untagged {
     None,
 }
 
+#[cfg(feature = "serde-compat")]
 #[test]
 fn test_serde_enum() {
     assert_eq!(
         SimpleEnum::decl(),
-        r#"export type SimpleEnum = {kind: "A", d: null} | {kind: "B", d: null};"#
+        r#"type SimpleEnum = { kind: "A" } | { kind: "B" };"#
     );
     assert_eq!(
         ComplexEnum::decl(),
-        r#"export type ComplexEnum = {kind: "A", data: null} | {kind: "B", data: {
+        r#"type ComplexEnum = { kind: "A" } | { kind: "B", data: {
     foo: string,
     bar: number,
-}} | {kind: "W", data: SimpleEnum} | {kind: "F", data: {
+} } | { kind: "W", data: SimpleEnum } | { kind: "F", data: {
     nested: SimpleEnum,
-}} | {kind: "T", data: [number, SimpleEnum]};"#
+} } | { kind: "T", data: [number, SimpleEnum] };"#
     );
 
     assert_eq!(
         Untagged::decl(),
-        r#"export type Untagged = string | number | null;"#
+        r#"type Untagged = string | number | null;"#
     )
 }
